#include "launcher.h"

#include <QDebug>
#include <QObject>
#include <QWidget>
#include <QDir>
#include <QCoreApplication>
#include <QString>
#include <QList>
#include <QJsonDocument>
#include <QEventLoop>

#include "profile_handler/forge1_12_2.h"
#include "../ui/mainwindow.h"
#include "../../resources/ui_mainwindow.h"


Launcher::Launcher(QObject *parent) : 
    QObject(parent), 
    process(new QProcess(this)), 
    get_args(new forge1_12_2()), 
    run_status(false),
    mainWindow(qobject_cast<MainWindow *>(parent)),
    app_dir(QDir(QCoreApplication::applicationDirPath()))
{
    if (!mainWindow) {
        qCritical() << "Failed to cast parent object to MainWindow";
    }
}


Launcher::~Launcher() {
    delete process;
    delete get_args;
}


void Launcher::startProcess(QString &username, QString &uuid, QString &access_token) {
    if (run_status) {
        qDebug() << "Process is already running";
        return;
    }
    run_status = true;

    selected_profile = this->mainWindow->ui->selected_profile->currentText();
    client_dir = app_dir.absolutePath() + "/" + selected_profile;
    json_settings = this->mainWindow->settings->loadFromJson(selected_profile);
    java_path = json_settings["java_path"].toString();
    RAM = json_settings["RAM"].toString();
    garbarge_collector = json_settings["garbarge_collector"].toString();

    if (selected_profile == "ZombieExtrieme") {
        arguments = get_args->getCrossPlatformArgs(RAM, username, uuid, access_token, garbarge_collector);
        java_path = java_path.isEmpty() ? app_dir.absolutePath() + "/java/zulu8.84.0.15-ca-jre8.0.442-linux_x64/bin/java" : java_path;
        RAM = RAM.isEmpty() ? "4096M" : RAM;
        garbarge_collector = garbarge_collector.isEmpty() ? "default_collector" : garbarge_collector;
        version = "1.12.2";
    }

    if (version == "1.12.2") {
        sync_dirs 
			<< "./java/zulu8.84.0.15-ca-jre8.0.442-linux_x64"
			<< "./versions/1.12.2"
			<< "./libraries/1.12.2"
			<< "./" + selected_profile;
		sync_files 
			<< "./PunkLauncher";
    }

    QEventLoop loop;
    connect(this, &Launcher::updateFinished, &loop, &QEventLoop::quit);
    update();
    loop.exec();

    process->setWorkingDirectory(client_dir);
    process->start(java_path, arguments);

    if (!process->waitForStarted()) {
        qDebug() << "Failed to start process";
        qDebug() << process->errorString();
        run_status = false;
        return;
    }

    connect(process,
        QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
        [this](int, QProcess::ExitStatus) { 
            this->mainWindow->show();
            run_status = false;
        });

    this->mainWindow->hide();
    qDebug() << "Process started successfully";
}


void Launcher::update() {
    QProcess *rsyncProcess = new QProcess(this);
    rsyncProcess->setWorkingDirectory(app_dir.absolutePath());

    connect(rsyncProcess, &QProcess::readyReadStandardOutput, [rsyncProcess, this]() {
        QString output = rsyncProcess->readAllStandardOutput();
        QStringList lines = output.split('\n');
        if (!lines.isEmpty()) {
            QString lastLine = lines.last().trimmed();
            int pos = lastLine.indexOf('(');
            if (pos != -1) {
                lastLine = lastLine.left(pos).trimmed();
            }
            if (!lastLine.isEmpty()) {
                this->mainWindow->ui->download_bar->setText(lastLine);
            }
        }
    });

    connect(rsyncProcess, &QProcess::readyReadStandardError, [rsyncProcess, this]() {
        QString errorOutput = rsyncProcess->readAllStandardError();
        this->mainWindow->ui->download_bar->setText(errorOutput);
        qDebug() << errorOutput;
    });

    QStringList rsyncArgs;
    rsyncArgs << "-azH" << "--info=progress2";
    for (const QString &dir : sync_dirs) { rsyncArgs << "--include" << dir + "/**"; }
    for (const QString &file : sync_files) { rsyncArgs << "--include" << file; }

    QStringList excludeFiles = {
        QDir::toNativeSeparators("./" + selected_profile + "/options.txt"),
        QDir::toNativeSeparators("./" + selected_profile + "/optionsof.txt"),
        QDir::toNativeSeparators("./" + selected_profile + "/optionsshaders.txt")
    };

    for (const QString &filePath : excludeFiles) {
        QFileInfo fileInfo(filePath);
        if (fileInfo.exists() && fileInfo.isFile()) {
            QString relativePath = QDir().relativeFilePath(fileInfo.absoluteFilePath());
            rsyncArgs << "--exclude" << relativePath;
        }
    }

    rsyncArgs << "--exclude" << "./**";
    rsyncArgs << "punkcraft.ru::PunkCraft/unix/uncompressed/" << "./";

    rsyncProcess->start("rsync", rsyncArgs);
    if (!rsyncProcess->waitForStarted()) {
        qDebug() << "Failed to start rsync process";
        qDebug() << rsyncProcess->errorString();
        emit updateFinished();  
        return;
    }

    connect(rsyncProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), [this, rsyncProcess]() {
        rsyncProcess->deleteLater();
        QProcess *rsyncProcess2 = new QProcess(this);
        rsyncProcess2->setWorkingDirectory(app_dir.absolutePath());

        QStringList modsArgs;
        modsArgs
			<< "-azH" << "--info=progress2" << "--delete-after"
			<< "--include=" + selected_profile
			<< "--include=" + selected_profile + "/mods/"
			<< "--include=" + selected_profile + "/mods/**"
			<< "--include=" + selected_profile + "/resourcepacks/"
			<< "--include=" + selected_profile + "/resourcepacks/**"
			<< "--exclude=*"
			<< "--exclude=" + selected_profile + "/*"
			<< "punkcraft.ru::PunkCraft/unix/uncompressed/" << "./";

        rsyncProcess2->start("rsync", modsArgs);

        connect(rsyncProcess2, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), [this, rsyncProcess2]() {
            rsyncProcess2->deleteLater();
            emit updateFinished();
            this->mainWindow->ui->download_bar->setText("");
			qDebug() << "Rsync process finished";
        });
    });
}
